# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing
import urllib.parse
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.jsonable_encoder import jsonable_encoder
from ...core.remove_none_from_dict import remove_none_from_dict
from ...errors.unprocessable_entity_error import UnprocessableEntityError
from ...types.api_key import ApiKey
from ...types.api_key_with_plain_key import ApiKeyWithPlainKey
from ...types.http_validation_error import HttpValidationError

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ApiKeysClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def read_api_keys(
        self, *, skip: typing.Optional[int] = None, limit: typing.Optional[int] = None
    ) -> typing.List[ApiKey]:
        """
        Retrieve all API keys for the current user.

        Args:
        ----
            db (AsyncSession): The database session.
            skip (int): Number of records to skip for pagination.
            limit (int): Maximum number of records to return.
            user (schemas.User): The current user.

        Returns:
        -------
            List[schemas.APIKey]: A list of API keys.

        Parameters:
            - skip: typing.Optional[int].

            - limit: typing.Optional[int].
        ---
        from airweave.client import AirweaveApi

        client = AirweaveApi(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.api_keys.read_api_keys()
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api_keys/"),
            params=remove_none_from_dict({"skip": skip, "limit": limit}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[ApiKey], _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create_api_key(self, *, expiration_date: typing.Optional[dt.datetime] = OMIT) -> ApiKeyWithPlainKey:
        """
        Create a new API key for the current user.

        Returns a temporary plain key for the user to store securely.
        This is not stored in the database.

        Args:
        ----
            db (AsyncSession): The database session.
            api_key_in (schemas.APIKeyCreate): The API key creation data.
            user (schemas.User): The current user.

        Returns:
        -------
            schemas.APIKeyWithPlainKey: The created API key object, including the key.

        Parameters:
            - expiration_date: typing.Optional[dt.datetime].
        ---
        from airweave.client import AirweaveApi

        client = AirweaveApi(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.api_keys.create_api_key()
        """
        _request: typing.Dict[str, typing.Any] = {}
        if expiration_date is not OMIT:
            _request["expiration_date"] = expiration_date
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api_keys/"),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ApiKeyWithPlainKey, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete_api_key(self, *, id: str) -> ApiKey:
        """
        Delete an API key.

        Args:
        ----
            db (AsyncSession): The database session.
            id (UUID): The ID of the API key.
            user (schemas.User): The current user.

        Returns:
        -------
            schemas.APIKey: The revoked API key object.

        Raises:
        ------
            HTTPException: If the API key is not found.

        Parameters:
            - id: str.
        ---
        from airweave.client import AirweaveApi

        client = AirweaveApi(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.api_keys.delete_api_key(
            id="id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api_keys/"),
            params=remove_none_from_dict({"id": id}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ApiKey, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def read_api_key(self, id: str) -> ApiKey:
        """
        Retrieve an API key by ID.

        Args:
        ----
            db (AsyncSession): The database session.
            id (UUID): The ID of the API key.
            user (schemas.User): The current user.

        Returns:
        -------
            schemas.APIKey: The API key object.

        Raises:
        ------
            HTTPException: If the API key is not found.

        Parameters:
            - id: str.
        ---
        from airweave.client import AirweaveApi

        client = AirweaveApi(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.api_keys.read_api_key(
            id="id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api_keys/{id}"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ApiKey, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncApiKeysClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def read_api_keys(
        self, *, skip: typing.Optional[int] = None, limit: typing.Optional[int] = None
    ) -> typing.List[ApiKey]:
        """
        Retrieve all API keys for the current user.

        Args:
        ----
            db (AsyncSession): The database session.
            skip (int): Number of records to skip for pagination.
            limit (int): Maximum number of records to return.
            user (schemas.User): The current user.

        Returns:
        -------
            List[schemas.APIKey]: A list of API keys.

        Parameters:
            - skip: typing.Optional[int].

            - limit: typing.Optional[int].
        ---
        from airweave.client import AsyncAirweaveApi

        client = AsyncAirweaveApi(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        await client.api_keys.read_api_keys()
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api_keys/"),
            params=remove_none_from_dict({"skip": skip, "limit": limit}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[ApiKey], _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create_api_key(self, *, expiration_date: typing.Optional[dt.datetime] = OMIT) -> ApiKeyWithPlainKey:
        """
        Create a new API key for the current user.

        Returns a temporary plain key for the user to store securely.
        This is not stored in the database.

        Args:
        ----
            db (AsyncSession): The database session.
            api_key_in (schemas.APIKeyCreate): The API key creation data.
            user (schemas.User): The current user.

        Returns:
        -------
            schemas.APIKeyWithPlainKey: The created API key object, including the key.

        Parameters:
            - expiration_date: typing.Optional[dt.datetime].
        ---
        from airweave.client import AsyncAirweaveApi

        client = AsyncAirweaveApi(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        await client.api_keys.create_api_key()
        """
        _request: typing.Dict[str, typing.Any] = {}
        if expiration_date is not OMIT:
            _request["expiration_date"] = expiration_date
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api_keys/"),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ApiKeyWithPlainKey, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete_api_key(self, *, id: str) -> ApiKey:
        """
        Delete an API key.

        Args:
        ----
            db (AsyncSession): The database session.
            id (UUID): The ID of the API key.
            user (schemas.User): The current user.

        Returns:
        -------
            schemas.APIKey: The revoked API key object.

        Raises:
        ------
            HTTPException: If the API key is not found.

        Parameters:
            - id: str.
        ---
        from airweave.client import AsyncAirweaveApi

        client = AsyncAirweaveApi(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        await client.api_keys.delete_api_key(
            id="id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api_keys/"),
            params=remove_none_from_dict({"id": id}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ApiKey, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def read_api_key(self, id: str) -> ApiKey:
        """
        Retrieve an API key by ID.

        Args:
        ----
            db (AsyncSession): The database session.
            id (UUID): The ID of the API key.
            user (schemas.User): The current user.

        Returns:
        -------
            schemas.APIKey: The API key object.

        Raises:
        ------
            HTTPException: If the API key is not found.

        Parameters:
            - id: str.
        ---
        from airweave.client import AsyncAirweaveApi

        client = AsyncAirweaveApi(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        await client.api_keys.read_api_key(
            id="id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api_keys/{id}"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ApiKey, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)
