# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.jsonable_encoder import jsonable_encoder
from ...core.remove_none_from_dict import remove_none_from_dict
from ...errors.unprocessable_entity_error import UnprocessableEntityError
from ...types.connection import Connection
from ...types.http_validation_error import HttpValidationError
from ...types.integration_type import IntegrationType

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ConnectionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_connection(self, connection_id: str) -> Connection:
        """
        Get a specific connection.

        Parameters:
            - connection_id: str.
        ---
        from airweave.client import AirweaveApi

        client = AirweaveApi(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.connections.get_connection(
            connection_id="connection_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"connections/detail/{connection_id}"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Connection, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def list_all_connected_integrations(self) -> typing.List[Connection]:
        """
        Get all active connections for the current user across all integration types.

        ---
        from airweave.client import AirweaveApi

        client = AirweaveApi(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.connections.list_all_connected_integrations()
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "connections/list"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[Connection], _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def list_connected_integrations(self, integration_type: IntegrationType) -> typing.List[Connection]:
        """
        Get all integrations of specified type connected to the current user.

        Args:
            integration_type (IntegrationType): The type of integration to get connections for.
            db (AsyncSession): The database session.
            user (schemas.User): The current user.

        Returns:
            list[schemas.Connection]: The list of connections.

        Parameters:
            - integration_type: IntegrationType.
        ---
        from airweave import IntegrationType
        from airweave.client import AirweaveApi

        client = AirweaveApi(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.connections.list_connected_integrations(
            integration_type=IntegrationType.SOURCE,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"connections/list/{integration_type}"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[Connection], _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def connect_integration(
        self,
        integration_type: IntegrationType,
        short_name: str,
        *,
        name: typing.Optional[str] = OMIT,
        config_fields: typing.Dict[str, typing.Any],
    ) -> Connection:
        """
        Connect to a source, destination, or embedding model.

        Expects a POST body with:
        ```json
        {
            "name": "required connection name",
            ... other config fields specific to the integration type ...
        }
        ```

        Parameters:
            - integration_type: IntegrationType.

            - short_name: str.

            - name: typing.Optional[str].

            - config_fields: typing.Dict[str, typing.Any].
        ---
        from airweave import IntegrationType
        from airweave.client import AirweaveApi

        client = AirweaveApi(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.connections.connect_integration(
            integration_type=IntegrationType.SOURCE,
            short_name="short_name",
            config_fields={"key": "value"},
        )
        """
        _request: typing.Dict[str, typing.Any] = {"config_fields": config_fields}
        if name is not OMIT:
            _request["name"] = name
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"connections/connect/{integration_type}/{short_name}"
            ),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Connection, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_connection_credentials(self, connection_id: str) -> typing.Dict[str, typing.Any]:
        """
        Get the credentials for a connection.

        Args:
            connection_id (UUID): The ID of the connection to get credentials for
            db (AsyncSession): The database session
            user (schemas.User): The current user

        Returns:
            dict: The credentials for the connection

        Parameters:
            - connection_id: str.
        ---
        from airweave.client import AirweaveApi

        client = AirweaveApi(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.connections.get_connection_credentials(
            connection_id="connection_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"connections/credentials/{connection_id}"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.Dict[str, typing.Any], _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete_connection(self, connection_id: str) -> Connection:
        """
        Delete a connection.

        Deletes the connection and integration credential.

        Args:
            db (AsyncSession): The database session
            connection_id (UUID): The ID of the connection to delete
            delete_syncs_and_data (bool): Whether to delete the associated syncs and data
            user (schemas.User): The current user

        Returns:
            schemas.Connection: The deleted connection

        Parameters:
            - connection_id: str.
        ---
        from airweave.client import AirweaveApi

        client = AirweaveApi(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.connections.delete_connection(
            connection_id="connection_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"connections/delete/source/{connection_id}"
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Connection, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def disconnect_source_connection(self, connection_id: str) -> Connection:
        """
        Disconnect from a source connection.

        Args:
            db (AsyncSession): The database session
            connection_id (UUID): The ID of the connection to disconnect
            user (schemas.User): The current user
        Returns:
            schemas.Connection: The disconnected connection

        Parameters:
            - connection_id: str.
        ---
        from airweave.client import AirweaveApi

        client = AirweaveApi(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.connections.disconnect_source_connection(
            connection_id="connection_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "PUT",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"connections/disconnect/source/{connection_id}"
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Connection, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def disconnect_destination_connection(self, connection_id: str) -> Connection:
        """
        Disconnect from a destination connection.

        Args:
            db (AsyncSession): The database session
            connection_id (UUID): The ID of the connection to disconnect
            user (schemas.User): The current user

        Returns:
            schemas.Connection: The disconnected connection

        Parameters:
            - connection_id: str.
        ---
        from airweave.client import AirweaveApi

        client = AirweaveApi(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.connections.disconnect_destination_connection(
            connection_id="connection_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "PUT",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"connections/disconnect/destination/{connection_id}"
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Connection, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_oauth_2_auth_url(self, *, short_name: str) -> str:
        """
        Get the OAuth2 authorization URL for a source.

        Parameters:
            - short_name: str.
        ---
        from airweave.client import AirweaveApi

        client = AirweaveApi(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.connections.get_oauth_2_auth_url(
            short_name="short_name",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "connections/oauth2/source/auth_url"),
            params=remove_none_from_dict({"short_name": short_name}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(str, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def send_oauth_2_code(self, *, short_name: str, code: str) -> Connection:
        """
        Send the OAuth2 authorization code for a source.

        This will:
        1. Get the OAuth2 settings for the source
        2. Exchange the authorization code for a token
        3. Create an integration credential with the token

        Parameters:
            - short_name: str.

            - code: str.
        ---
        from airweave.client import AirweaveApi

        client = AirweaveApi(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.connections.send_oauth_2_code(
            short_name="short_name",
            code="code",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "connections/oauth2/source/code"),
            json=jsonable_encoder({"short_name": short_name, "code": code}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Connection, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def send_oauth_2_white_label_code(self, white_label_id: str, *, request: str) -> Connection:
        """
        Exchange the OAuth2 authorization code for a white label integration.

        Parameters:
            - white_label_id: str.

            - request: str.
        ---
        from airweave.client import AirweaveApi

        client = AirweaveApi(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.connections.send_oauth_2_white_label_code(
            white_label_id="white_label_id",
            request="string",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"connections/oauth2/white-label/{white_label_id}/code"
            ),
            json=jsonable_encoder(request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Connection, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_oauth_2_white_label_auth_url(self, white_label_id: str) -> str:
        """
        Get the OAuth2 authorization URL for a white label integration.

        Parameters:
            - white_label_id: str.
        ---
        from airweave.client import AirweaveApi

        client = AirweaveApi(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.connections.get_oauth_2_white_label_auth_url(
            white_label_id="white_label_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"connections/oauth2/white-label/{white_label_id}/auth_url"
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(str, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncConnectionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_connection(self, connection_id: str) -> Connection:
        """
        Get a specific connection.

        Parameters:
            - connection_id: str.
        ---
        from airweave.client import AsyncAirweaveApi

        client = AsyncAirweaveApi(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        await client.connections.get_connection(
            connection_id="connection_id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"connections/detail/{connection_id}"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Connection, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def list_all_connected_integrations(self) -> typing.List[Connection]:
        """
        Get all active connections for the current user across all integration types.

        ---
        from airweave.client import AsyncAirweaveApi

        client = AsyncAirweaveApi(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        await client.connections.list_all_connected_integrations()
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "connections/list"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[Connection], _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def list_connected_integrations(self, integration_type: IntegrationType) -> typing.List[Connection]:
        """
        Get all integrations of specified type connected to the current user.

        Args:
            integration_type (IntegrationType): The type of integration to get connections for.
            db (AsyncSession): The database session.
            user (schemas.User): The current user.

        Returns:
            list[schemas.Connection]: The list of connections.

        Parameters:
            - integration_type: IntegrationType.
        ---
        from airweave import IntegrationType
        from airweave.client import AsyncAirweaveApi

        client = AsyncAirweaveApi(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        await client.connections.list_connected_integrations(
            integration_type=IntegrationType.SOURCE,
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"connections/list/{integration_type}"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[Connection], _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def connect_integration(
        self,
        integration_type: IntegrationType,
        short_name: str,
        *,
        name: typing.Optional[str] = OMIT,
        config_fields: typing.Dict[str, typing.Any],
    ) -> Connection:
        """
        Connect to a source, destination, or embedding model.

        Expects a POST body with:
        ```json
        {
            "name": "required connection name",
            ... other config fields specific to the integration type ...
        }
        ```

        Parameters:
            - integration_type: IntegrationType.

            - short_name: str.

            - name: typing.Optional[str].

            - config_fields: typing.Dict[str, typing.Any].
        ---
        from airweave import IntegrationType
        from airweave.client import AsyncAirweaveApi

        client = AsyncAirweaveApi(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        await client.connections.connect_integration(
            integration_type=IntegrationType.SOURCE,
            short_name="short_name",
            config_fields={"key": "value"},
        )
        """
        _request: typing.Dict[str, typing.Any] = {"config_fields": config_fields}
        if name is not OMIT:
            _request["name"] = name
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"connections/connect/{integration_type}/{short_name}"
            ),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Connection, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_connection_credentials(self, connection_id: str) -> typing.Dict[str, typing.Any]:
        """
        Get the credentials for a connection.

        Args:
            connection_id (UUID): The ID of the connection to get credentials for
            db (AsyncSession): The database session
            user (schemas.User): The current user

        Returns:
            dict: The credentials for the connection

        Parameters:
            - connection_id: str.
        ---
        from airweave.client import AsyncAirweaveApi

        client = AsyncAirweaveApi(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        await client.connections.get_connection_credentials(
            connection_id="connection_id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"connections/credentials/{connection_id}"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.Dict[str, typing.Any], _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete_connection(self, connection_id: str) -> Connection:
        """
        Delete a connection.

        Deletes the connection and integration credential.

        Args:
            db (AsyncSession): The database session
            connection_id (UUID): The ID of the connection to delete
            delete_syncs_and_data (bool): Whether to delete the associated syncs and data
            user (schemas.User): The current user

        Returns:
            schemas.Connection: The deleted connection

        Parameters:
            - connection_id: str.
        ---
        from airweave.client import AsyncAirweaveApi

        client = AsyncAirweaveApi(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        await client.connections.delete_connection(
            connection_id="connection_id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"connections/delete/source/{connection_id}"
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Connection, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def disconnect_source_connection(self, connection_id: str) -> Connection:
        """
        Disconnect from a source connection.

        Args:
            db (AsyncSession): The database session
            connection_id (UUID): The ID of the connection to disconnect
            user (schemas.User): The current user
        Returns:
            schemas.Connection: The disconnected connection

        Parameters:
            - connection_id: str.
        ---
        from airweave.client import AsyncAirweaveApi

        client = AsyncAirweaveApi(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        await client.connections.disconnect_source_connection(
            connection_id="connection_id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "PUT",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"connections/disconnect/source/{connection_id}"
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Connection, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def disconnect_destination_connection(self, connection_id: str) -> Connection:
        """
        Disconnect from a destination connection.

        Args:
            db (AsyncSession): The database session
            connection_id (UUID): The ID of the connection to disconnect
            user (schemas.User): The current user

        Returns:
            schemas.Connection: The disconnected connection

        Parameters:
            - connection_id: str.
        ---
        from airweave.client import AsyncAirweaveApi

        client = AsyncAirweaveApi(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        await client.connections.disconnect_destination_connection(
            connection_id="connection_id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "PUT",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"connections/disconnect/destination/{connection_id}"
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Connection, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_oauth_2_auth_url(self, *, short_name: str) -> str:
        """
        Get the OAuth2 authorization URL for a source.

        Parameters:
            - short_name: str.
        ---
        from airweave.client import AsyncAirweaveApi

        client = AsyncAirweaveApi(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        await client.connections.get_oauth_2_auth_url(
            short_name="short_name",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "connections/oauth2/source/auth_url"),
            params=remove_none_from_dict({"short_name": short_name}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(str, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def send_oauth_2_code(self, *, short_name: str, code: str) -> Connection:
        """
        Send the OAuth2 authorization code for a source.

        This will:
        1. Get the OAuth2 settings for the source
        2. Exchange the authorization code for a token
        3. Create an integration credential with the token

        Parameters:
            - short_name: str.

            - code: str.
        ---
        from airweave.client import AsyncAirweaveApi

        client = AsyncAirweaveApi(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        await client.connections.send_oauth_2_code(
            short_name="short_name",
            code="code",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "connections/oauth2/source/code"),
            json=jsonable_encoder({"short_name": short_name, "code": code}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Connection, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def send_oauth_2_white_label_code(self, white_label_id: str, *, request: str) -> Connection:
        """
        Exchange the OAuth2 authorization code for a white label integration.

        Parameters:
            - white_label_id: str.

            - request: str.
        ---
        from airweave.client import AsyncAirweaveApi

        client = AsyncAirweaveApi(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        await client.connections.send_oauth_2_white_label_code(
            white_label_id="white_label_id",
            request="string",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"connections/oauth2/white-label/{white_label_id}/code"
            ),
            json=jsonable_encoder(request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Connection, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_oauth_2_white_label_auth_url(self, white_label_id: str) -> str:
        """
        Get the OAuth2 authorization URL for a white label integration.

        Parameters:
            - white_label_id: str.
        ---
        from airweave.client import AsyncAirweaveApi

        client = AsyncAirweaveApi(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        await client.connections.get_oauth_2_white_label_auth_url(
            white_label_id="white_label_id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"connections/oauth2/white-label/{white_label_id}/auth_url"
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(str, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)
