# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...errors.unprocessable_entity_error import UnprocessableEntityError
from ...types.embedding_model import EmbeddingModel
from ...types.embedding_model_with_config_fields import EmbeddingModelWithConfigFields
from ...types.http_validation_error import HttpValidationError

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class EmbeddingModelsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def read_embedding_model(self, short_name: str) -> EmbeddingModelWithConfigFields:
        """
        Get embedding model by id.

        Args:
        ----
            db (AsyncSession): The database session.
            short_name (str): The short name of the embedding model.
            user (schemas.User): The current user.

        Returns:
        -------
            schemas.EmbeddingModel: The embedding model object.

        Parameters:
            - short_name: str.
        ---
        from airweave.client import AirweaveApi

        client = AirweaveApi(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.embedding_models.read_embedding_model(
            short_name="short_name",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"embedding_models/detail/{short_name}"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(EmbeddingModelWithConfigFields, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def read_embedding_models(self) -> typing.List[EmbeddingModel]:
        """
        Get all embedding models.

        Args:
        ----
            db (AsyncSession): The database session.
            user (schemas.User): The current user.

        Returns:
        -------
            list[schemas.EmbeddingModel]: The list of embedding models.

        ---
        from airweave.client import AirweaveApi

        client = AirweaveApi(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.embedding_models.read_embedding_models()
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "embedding_models/list"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[EmbeddingModel], _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncEmbeddingModelsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def read_embedding_model(self, short_name: str) -> EmbeddingModelWithConfigFields:
        """
        Get embedding model by id.

        Args:
        ----
            db (AsyncSession): The database session.
            short_name (str): The short name of the embedding model.
            user (schemas.User): The current user.

        Returns:
        -------
            schemas.EmbeddingModel: The embedding model object.

        Parameters:
            - short_name: str.
        ---
        from airweave.client import AsyncAirweaveApi

        client = AsyncAirweaveApi(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        await client.embedding_models.read_embedding_model(
            short_name="short_name",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"embedding_models/detail/{short_name}"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(EmbeddingModelWithConfigFields, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def read_embedding_models(self) -> typing.List[EmbeddingModel]:
        """
        Get all embedding models.

        Args:
        ----
            db (AsyncSession): The database session.
            user (schemas.User): The current user.

        Returns:
        -------
            list[schemas.EmbeddingModel]: The list of embedding models.

        ---
        from airweave.client import AsyncAirweaveApi

        client = AsyncAirweaveApi(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        await client.embedding_models.read_embedding_models()
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "embedding_models/list"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[EmbeddingModel], _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)
