# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.jsonable_encoder import jsonable_encoder
from ...core.remove_none_from_dict import remove_none_from_dict
from ...errors.unprocessable_entity_error import UnprocessableEntityError
from ...types.http_validation_error import HttpValidationError
from ...types.sync import Sync
from ...types.sync_job import SyncJob

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class SyncClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list_syncs(self, *, skip: typing.Optional[int] = None, limit: typing.Optional[int] = None) -> typing.List[Sync]:
        """
        List all syncs for the current user.

        Parameters:
            - skip: typing.Optional[int].

            - limit: typing.Optional[int].
        ---
        from airweave.client import AirweaveApi

        client = AirweaveApi(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.sync.list_syncs()
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "sync/"),
            params=remove_none_from_dict({"skip": skip, "limit": limit}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[Sync], _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create_sync(
        self,
        *,
        name: str,
        description: typing.Optional[str] = OMIT,
        source_connection_id: str,
        destination_connection_id: typing.Optional[str] = OMIT,
        embedding_model_connection_id: typing.Optional[str] = OMIT,
        cron_schedule: typing.Optional[str] = OMIT,
        white_label_id: typing.Optional[str] = OMIT,
        white_label_user_identifier: typing.Optional[str] = OMIT,
        sync_metadata: typing.Optional[typing.Dict[str, typing.Any]] = OMIT,
        run_immediately: typing.Optional[bool] = OMIT,
    ) -> Sync:
        """
        Create a new sync configuration.

        Parameters:
            - name: str.

            - description: typing.Optional[str].

            - source_connection_id: str.

            - destination_connection_id: typing.Optional[str].

            - embedding_model_connection_id: typing.Optional[str].

            - cron_schedule: typing.Optional[str].

            - white_label_id: typing.Optional[str].

            - white_label_user_identifier: typing.Optional[str].

            - sync_metadata: typing.Optional[typing.Dict[str, typing.Any]].

            - run_immediately: typing.Optional[bool].
        ---
        from airweave.client import AirweaveApi

        client = AirweaveApi(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.sync.create_sync(
            name="name",
            source_connection_id="source_connection_id",
        )
        """
        _request: typing.Dict[str, typing.Any] = {"name": name, "source_connection_id": source_connection_id}
        if description is not OMIT:
            _request["description"] = description
        if destination_connection_id is not OMIT:
            _request["destination_connection_id"] = destination_connection_id
        if embedding_model_connection_id is not OMIT:
            _request["embedding_model_connection_id"] = embedding_model_connection_id
        if cron_schedule is not OMIT:
            _request["cron_schedule"] = cron_schedule
        if white_label_id is not OMIT:
            _request["white_label_id"] = white_label_id
        if white_label_user_identifier is not OMIT:
            _request["white_label_user_identifier"] = white_label_user_identifier
        if sync_metadata is not OMIT:
            _request["sync_metadata"] = sync_metadata
        if run_immediately is not OMIT:
            _request["run_immediately"] = run_immediately
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "sync/"),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Sync, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_sync(self, sync_id: str) -> Sync:
        """
        Get a specific sync by ID.

        Parameters:
            - sync_id: str.
        ---
        from airweave.client import AirweaveApi

        client = AirweaveApi(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.sync.get_sync(
            sync_id="sync_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"sync/{sync_id}"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Sync, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete_sync(self, sync_id: str, *, delete_data: typing.Optional[bool] = None) -> Sync:
        """
        Delete a sync configuration and optionally its associated data.

        Parameters:
            - sync_id: str.

            - delete_data: typing.Optional[bool].
        ---
        from airweave.client import AirweaveApi

        client = AirweaveApi(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.sync.delete_sync(
            sync_id="sync_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"sync/{sync_id}"),
            params=remove_none_from_dict({"delete_data": delete_data}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Sync, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def run_sync(self, sync_id: str) -> SyncJob:
        """
        Trigger a sync run.

        Parameters:
            - sync_id: str.
        ---
        from airweave.client import AirweaveApi

        client = AirweaveApi(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.sync.run_sync(
            sync_id="sync_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"sync/{sync_id}/run"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(SyncJob, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def list_sync_jobs(self, sync_id: str) -> typing.List[SyncJob]:
        """
        List all jobs for a specific sync.

        Parameters:
            - sync_id: str.
        ---
        from airweave.client import AirweaveApi

        client = AirweaveApi(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.sync.list_sync_jobs(
            sync_id="sync_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"sync/{sync_id}/jobs"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[SyncJob], _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_sync_job(self, job_id: str, *, sync_id: str) -> SyncJob:
        """
        Get details of a specific sync job.

        Parameters:
            - job_id: str.

            - sync_id: str.
        ---
        from airweave.client import AirweaveApi

        client = AirweaveApi(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.sync.get_sync_job(
            job_id="job_id",
            sync_id="sync_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"sync/job/{job_id}"),
            params=remove_none_from_dict({"sync_id": sync_id}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(SyncJob, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def subscribe_sync_job(self, job_id: str) -> typing.Any:
        """
        Server-Sent Events (SSE) endpoint to subscribe to a sync job's progress.

        Parameters:
            - job_id: str.
        ---
        from airweave.client import AirweaveApi

        client = AirweaveApi(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.sync.subscribe_sync_job(
            job_id="job_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"sync/job/{job_id}/subscribe"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.Any, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncSyncClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list_syncs(
        self, *, skip: typing.Optional[int] = None, limit: typing.Optional[int] = None
    ) -> typing.List[Sync]:
        """
        List all syncs for the current user.

        Parameters:
            - skip: typing.Optional[int].

            - limit: typing.Optional[int].
        ---
        from airweave.client import AsyncAirweaveApi

        client = AsyncAirweaveApi(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        await client.sync.list_syncs()
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "sync/"),
            params=remove_none_from_dict({"skip": skip, "limit": limit}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[Sync], _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create_sync(
        self,
        *,
        name: str,
        description: typing.Optional[str] = OMIT,
        source_connection_id: str,
        destination_connection_id: typing.Optional[str] = OMIT,
        embedding_model_connection_id: typing.Optional[str] = OMIT,
        cron_schedule: typing.Optional[str] = OMIT,
        white_label_id: typing.Optional[str] = OMIT,
        white_label_user_identifier: typing.Optional[str] = OMIT,
        sync_metadata: typing.Optional[typing.Dict[str, typing.Any]] = OMIT,
        run_immediately: typing.Optional[bool] = OMIT,
    ) -> Sync:
        """
        Create a new sync configuration.

        Parameters:
            - name: str.

            - description: typing.Optional[str].

            - source_connection_id: str.

            - destination_connection_id: typing.Optional[str].

            - embedding_model_connection_id: typing.Optional[str].

            - cron_schedule: typing.Optional[str].

            - white_label_id: typing.Optional[str].

            - white_label_user_identifier: typing.Optional[str].

            - sync_metadata: typing.Optional[typing.Dict[str, typing.Any]].

            - run_immediately: typing.Optional[bool].
        ---
        from airweave.client import AsyncAirweaveApi

        client = AsyncAirweaveApi(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        await client.sync.create_sync(
            name="name",
            source_connection_id="source_connection_id",
        )
        """
        _request: typing.Dict[str, typing.Any] = {"name": name, "source_connection_id": source_connection_id}
        if description is not OMIT:
            _request["description"] = description
        if destination_connection_id is not OMIT:
            _request["destination_connection_id"] = destination_connection_id
        if embedding_model_connection_id is not OMIT:
            _request["embedding_model_connection_id"] = embedding_model_connection_id
        if cron_schedule is not OMIT:
            _request["cron_schedule"] = cron_schedule
        if white_label_id is not OMIT:
            _request["white_label_id"] = white_label_id
        if white_label_user_identifier is not OMIT:
            _request["white_label_user_identifier"] = white_label_user_identifier
        if sync_metadata is not OMIT:
            _request["sync_metadata"] = sync_metadata
        if run_immediately is not OMIT:
            _request["run_immediately"] = run_immediately
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "sync/"),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Sync, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_sync(self, sync_id: str) -> Sync:
        """
        Get a specific sync by ID.

        Parameters:
            - sync_id: str.
        ---
        from airweave.client import AsyncAirweaveApi

        client = AsyncAirweaveApi(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        await client.sync.get_sync(
            sync_id="sync_id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"sync/{sync_id}"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Sync, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete_sync(self, sync_id: str, *, delete_data: typing.Optional[bool] = None) -> Sync:
        """
        Delete a sync configuration and optionally its associated data.

        Parameters:
            - sync_id: str.

            - delete_data: typing.Optional[bool].
        ---
        from airweave.client import AsyncAirweaveApi

        client = AsyncAirweaveApi(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        await client.sync.delete_sync(
            sync_id="sync_id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"sync/{sync_id}"),
            params=remove_none_from_dict({"delete_data": delete_data}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Sync, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def run_sync(self, sync_id: str) -> SyncJob:
        """
        Trigger a sync run.

        Parameters:
            - sync_id: str.
        ---
        from airweave.client import AsyncAirweaveApi

        client = AsyncAirweaveApi(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        await client.sync.run_sync(
            sync_id="sync_id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"sync/{sync_id}/run"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(SyncJob, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def list_sync_jobs(self, sync_id: str) -> typing.List[SyncJob]:
        """
        List all jobs for a specific sync.

        Parameters:
            - sync_id: str.
        ---
        from airweave.client import AsyncAirweaveApi

        client = AsyncAirweaveApi(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        await client.sync.list_sync_jobs(
            sync_id="sync_id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"sync/{sync_id}/jobs"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[SyncJob], _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_sync_job(self, job_id: str, *, sync_id: str) -> SyncJob:
        """
        Get details of a specific sync job.

        Parameters:
            - job_id: str.

            - sync_id: str.
        ---
        from airweave.client import AsyncAirweaveApi

        client = AsyncAirweaveApi(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        await client.sync.get_sync_job(
            job_id="job_id",
            sync_id="sync_id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"sync/job/{job_id}"),
            params=remove_none_from_dict({"sync_id": sync_id}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(SyncJob, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def subscribe_sync_job(self, job_id: str) -> typing.Any:
        """
        Server-Sent Events (SSE) endpoint to subscribe to a sync job's progress.

        Parameters:
            - job_id: str.
        ---
        from airweave.client import AsyncAirweaveApi

        client = AsyncAirweaveApi(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        await client.sync.subscribe_sync_job(
            job_id="job_id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"sync/job/{job_id}/subscribe"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.Any, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)
